@@ -1,9 +1,6 @@
-use crate::{
-    arrayview::ArrayView,
-    structs::{IndexStruct, VariadicRef, VariadicRefFactory, VariadicStruct},
-};
+use crate::structs::{IndexStruct, VariadicRef, VariadicRefFactory, VariadicStruct};
 
-use std::{fmt, iter, marker, ops::RangeBounds};
+use std::{fmt, iter, marker};
 
 /// A read-only view on a multivector.
 ///
@@ -11,14 +8,21 @@ use std::{fmt, iter, marker, ops::RangeBounds};
 /// [`MultiVector`].
 ///
 /// [`MultiVector`]: struct.MultiVector.html
-#[derive(Clone)]
 pub struct MultiArrayView<'a, Ts>
 where
     Ts: VariadicRefFactory,
 {
-    index: ArrayView<'a, <Ts as VariadicStruct<'a>>::Index>,
+    index: &'a [Ts::Index],
     data: &'a [u8],
-    _phantom: marker::PhantomData<Ts>,
+}
+
+impl<'a, Ts> Clone for MultiArrayView<'a, Ts>
+where
+    Ts: VariadicRefFactory,
+{
+    fn clone(&self) -> Self {
+        Self { ..*self }
+    }
 }
 
 impl<'a, Ts> MultiArrayView<'a, Ts>
@@ -28,15 +32,8 @@ where
     /// Creates a new `MultiArrayView` to the data at the given address.
     ///
     /// The returned array view does not own the data.
-    pub fn new(
-        index: ArrayView<'a, <Ts as VariadicStruct<'a>>::Index>,
-        data_mem_descr: &'a [u8],
-    ) -> Self {
-        Self {
-            index,
-            data: &data_mem_descr,
-            _phantom: marker::PhantomData,
-        }
+    pub fn new(index: &'a [Ts::Index], data: &'a [u8]) -> Self {
+        Self { index, data }
     }
 
     /// Number of indexed items in the array.
@@ -59,7 +56,7 @@ where
     ///
     /// Panics if index is greater than or equal to `MultiArrayView::len()`.
     pub fn at(&self, index: usize) -> MultiArrayViewItemIter<'a, Ts> {
-        let range = <<Ts as VariadicStruct<'a>>::Index as IndexStruct>::range(self.index.at(index));
+        let range = <Ts::Index>::range(&self.index[index]);
         MultiArrayViewItemIter {
             data: &self.data[range],
             _phantom: marker::PhantomData,
@@ -71,8 +68,11 @@ where
     /// # Panics
     ///
     /// Panics if the range is outside of bounds of array view.
-    pub fn slice<R: RangeBounds<usize>>(&self, range: R) -> Self {
-        Self::new(self.index.slice(range), self.data)
+    pub fn slice<R: std::slice::SliceIndex<[Ts::Index], Output = [Ts::Index]>>(
+        &self,
+        range: R,
+    ) -> Self {
+        Self::new(&self.index[range], self.data)
     }
 
     /// Returns an iterator through the indexed items of the array.
@@ -251,49 +251,25 @@ where
 #[allow(dead_code)]
 mod tests {
     use super::*;
-    use crate::{memstorage::MemoryResourceStorage, storage::create_multi_vector};
+    use crate::{
+        memstorage::MemoryResourceStorage,
+        storage::create_multi_vector,
+        test::{Ab, AbRef},
+    };
 
-    define_index!(Idx, RefIdx, RefMutIdx, "some_idx_schema", 4, 32);
-
-    define_struct!(
-        Value,
-        RefValue,
-        RefMutValue,
-        "no_schema",
-        3,
-        (value, set_value, u32, u32, 0, 24)
-    );
-
-    define_struct!(
-        Point,
-        RefPoint,
-        RefMutPoint,
-        "no_schema",
-        4,
-        (x, set_x, u32, u32, 0, 24),
-        (y, set_y, u32, u32, 0, 24)
-    );
-
-    define_variadic_struct!(Variant, RefVariant, BuilderVariant, Idx,
-        0 => (Value, Value, add_value),
-        1 => (Point, Point, add_point) );
-
-    fn create_view<'a>(
-        storage: &'a MemoryResourceStorage,
-        size: usize,
-    ) -> MultiArrayView<'a, Variant> {
-        let mut mv = create_multi_vector::<Variant>(&*storage, "multivector", "Some schema")
+    fn create_view<'a>(storage: &'a MemoryResourceStorage, size: usize) -> MultiArrayView<'a, Ab> {
+        let mut mv = create_multi_vector::<Ab>(&*storage, "multivector", "Some schema")
             .expect("failed to create MultiVector");
 
         for i in 0..size {
             let mut item = mv.grow().expect("grow failed");
 
-            let mut a = item.add_value();
-            a.set_value(i as u32);
+            let b = item.add_b();
+            b.set_id(i as u32);
 
-            let mut b = item.add_point();
-            b.set_x((i + size) as u32);
-            b.set_y((i + 2 * size) as u32);
+            let a = item.add_a();
+            a.set_x((i + size) as u32);
+            a.set_y((i + 2 * size) as u32);
         }
 
         mv.close().expect("close failed")
@@ -305,7 +281,7 @@ mod tests {
         let view = create_view(&storage, 10);
 
         let value = |mut iter: MultiArrayViewItemIter<_>| match iter.next().unwrap() {
-            RefVariant::Value(v) => v.value(),
+            AbRef::B(v) => v.id(),
             otherwise => panic!("unexpected value: {:?}", otherwise),
         };
 
@@ -324,7 +300,7 @@ mod tests {
         let view = create_view(&storage, 10);
 
         let value = |mut iter: MultiArrayViewItemIter<_>| match iter.next().unwrap() {
-            RefVariant::Value(v) => v.value(),
+            AbRef::B(v) => v.id(),
             otherwise => panic!("unexpected value: {:?}", otherwise),
         };
 
@@ -337,16 +313,16 @@ mod tests {
         let storage = MemoryResourceStorage::new("/root/resources");
         let view = create_view(&storage, 100);
         let content = " { len: 100, data: [\
-                       (0, [Value { value: 0 }, Point { x: 200, y: 200 }]), \
-                       (1, [Value { value: 1 }, Point { x: 201, y: 201 }]), \
-                       (2, [Value { value: 2 }, Point { x: 202, y: 202 }]), \
-                       (3, [Value { value: 3 }, Point { x: 203, y: 203 }]), \
-                       (4, [Value { value: 4 }, Point { x: 204, y: 204 }]), \
-                       (5, [Value { value: 5 }, Point { x: 205, y: 205 }]), \
-                       (6, [Value { value: 6 }, Point { x: 206, y: 206 }]), \
-                       (7, [Value { value: 7 }, Point { x: 207, y: 207 }]), \
-                       (8, [Value { value: 8 }, Point { x: 208, y: 208 }]), \
-                       (9, [Value { value: 9 }, Point { x: 209, y: 209 }])]... }";
+                       (0, [B { id: 0 }, A { x: 100, y: 200, e: Value }]), \
+                       (1, [B { id: 1 }, A { x: 101, y: 201, e: Value }]), \
+                       (2, [B { id: 2 }, A { x: 102, y: 202, e: Value }]), \
+                       (3, [B { id: 3 }, A { x: 103, y: 203, e: Value }]), \
+                       (4, [B { id: 4 }, A { x: 104, y: 204, e: Value }]), \
+                       (5, [B { id: 5 }, A { x: 105, y: 205, e: Value }]), \
+                       (6, [B { id: 6 }, A { x: 106, y: 206, e: Value }]), \
+                       (7, [B { id: 7 }, A { x: 107, y: 207, e: Value }]), \
+                       (8, [B { id: 8 }, A { x: 108, y: 208, e: Value }]), \
+                       (9, [B { id: 9 }, A { x: 109, y: 209, e: Value }])]... }";
 
         assert_eq!(
             format!("{:?}", view),
@@ -358,7 +334,7 @@ mod tests {
         );
         assert_eq!(
             format!("{:?}", view.at(5)),
-            "MultiArrayViewItemIter { data: [Value { value: 5 }, Point { x: 205, y: 205 }] }"
+            "MultiArrayViewItemIter { data: [B { id: 5 }, A { x: 105, y: 205, e: Value }] }"
         );
         let mut iter = view.iter();
         for _ in 0..99 {
@@ -367,7 +343,7 @@ mod tests {
         assert_eq!(
             format!("{:?}", iter),
             "MultiArrayViewIter { len: 1, data: \
-             [(0, [Value { value: 99 }, Point { x: 299, y: 299 }])] }"
+             [(0, [B { id: 99 }, A { x: 199, y: 299, e: Value }])] }"
         );
     }
 
